package cn.com.qmth.export;

import java.io.File;
import java.io.IOException;
import java.sql.Connection;
import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import oracle.jdbc.driver.OracleDriver;

public class ExportPaperByAllQuestions {
	private static Logger logger = LogManager.getLogger(Export.class);
	private static String excelDir;
	private static String quesDir;
	private static List<CourseInfo> codes;

	private static Map<String, String> quesTypes = new HashMap<String, String>();
	static {
		quesTypes.put("11", "单选题");
		quesTypes.put("13", "填空题");
		quesTypes.put("20", "判断题");
		quesTypes.put("69", "名词解释");
		quesTypes.put("12", "多选题");
		quesTypes.put("60", "问答题");
		quesTypes.put("42", "复合题");
		quesTypes.put("14", "选词填空");
		quesTypes.put("15", "完形填空");
		quesTypes.put("61", "分析题");

	}

	public static void main(String[] args) {
		logger.debug("export start...");
		try {
			dispose();
		} catch (Exception e) {
			logger.error(e.getCause(), e);
		}
		logger.debug("export end...");
	}

	public static void dispose() throws InvalidFormatException, IOException, SQLException {
		File directory = new File("");
		excelDir = directory.getAbsolutePath() + "\\excelDir\\";
		quesDir = excelDir + "quesDir\\";
		File excelFolder = new File(excelDir);
		if (excelFolder.exists()) {
			FileUtil.clearDirectory(excelDir);
		} else {
			excelFolder.mkdir();
		}
		File quesFolder = new File(quesDir);
		quesFolder.mkdir();
		codes = readCourseCode();
		if (codes == null || codes.size() == 0) {
			logger.debug("no data export");
			return;
		}

		Connection connect = null;

		try {
			Driver driver = new OracleDriver();
			DriverManager.deregisterDriver(driver);

			Properties pro = new Properties();
			pro.put("user", "qspad");
			pro.put("password", "qspad1");
			connect = driver.connect("jdbc:oracle:thin:@202.114.196.115:1521:qspad", pro);
			fillCourseInfo(connect);
			for (CourseInfo code : codes) {
				exportPaper(connect, code);
			}
		} finally {
			if (connect != null) {
				connect.close();
			}
		}
	}

	private static List<CourseInfo> readCourseCode() throws InvalidFormatException, IOException {
		File directory = new File("");
		List<CourseInfo> list = new ArrayList<CourseInfo>();
		XSSFWorkbook wb = null;
		try {
			wb = new XSSFWorkbook(directory.getAbsolutePath() + "\\CourseCode.xlsx");
			XSSFSheet sheet = wb.getSheetAt(0);
			int rows = sheet.getLastRowNum();
			for (int i = 1; i <= rows; i++) {
				XSSFRow row = sheet.getRow(i);
				list.add(
						new CourseInfo(null, row.getCell(0).getStringCellValue(), row.getCell(1).getStringCellValue()));
			}
		} finally {
			if (wb != null) {
				wb.close();
			}
		}
		return list;
	}

	private static void fillCourseInfo(Connection connect) throws SQLException, IOException {
		Map<String, CourseInfo> courseMaps = new HashMap();
		for (CourseInfo code : codes) {
			courseMaps.put(code.getCode(), code);
		}
		PreparedStatement preState = null;
		ResultSet resultSet = null;

		try {
			String sql = "SELECT c.id,c.name,c.code FROM PE_TCH_COURSE c WHERE c.code in (";
			for (CourseInfo c : codes) {
				sql = sql + "'" + c.getCode() + "',";
			}
			sql = sql + "'607816a3f8461343b1ea5a04')";
			preState = connect.prepareStatement(sql);

			resultSet = preState.executeQuery();
			List<CourseInfo> list = new ArrayList<CourseInfo>();
			while (resultSet.next()) {
				String code=resultSet.getString("code");
				CourseInfo tq = new CourseInfo();
				tq.setId(resultSet.getString("id"));
				tq.setName(courseMaps.get(code).getName());
				tq.setCode(code);
				list.add(tq);
			}
			codes = list;
		} finally {
			if (resultSet != null) {
				resultSet.close();
			}
			if (preState != null) {
				preState.close();
			}
		}
	}

	private static void exportPaper(Connection connect, CourseInfo course) throws SQLException, IOException {
		PreparedStatement preState = null;
		ResultSet resultSet = null;

		try {
			TestPaper tp = new TestPaper();
			tp.setId("none");
			tp.setName(course.getName() + "试题库试卷");
			tp.setTestPaperContentId("none");
			tp.setScore(0.0);
			List<SheetData> sheets = new ArrayList<SheetData>();
			List<String> paperheader = new ArrayList<String>();
			paperheader.addAll(Arrays.asList(new String[] { "试卷ID", "试卷名称", "试卷内容ID", "试卷满分" }));
			fillPaperData(sheets, tp, paperheader);
			exportQuestion(connect, course.getId(), sheets);
			fillScore(sheets);
			ExportUtils.exportExcel(excelDir, course.getCode(), sheets);
		} finally {
			if (resultSet != null) {
				resultSet.close();
			}
			if (preState != null) {
				preState.close();
			}
		}
	}

	private static void fillScore(List<SheetData> sheets) {
		SheetData s1 = sheets.get(1);
		List<Object[]> data1 = s1.getData();
		Map<String, Integer> sc = new HashMap<String, Integer>();
		for (Object[] obs : data1) {
			String pid = (String) obs[5];
			if (!"00000000000000000000000000000000".equals(pid)) {
				Integer scr = sc.get(pid);
				if (scr == null) {
					sc.put(pid, 1);
				} else {
					sc.put(pid, scr + 1);
				}
			}
		}
		for (Object[] obs : data1) {
			String id = (String) obs[0];
			if (sc.get(id) != null) {
				obs[4] = sc.get(id) + ".0";
			}
		}

		SheetData s0 = sheets.get(0);
		List<Object[]> data0 = s0.getData();
		data0.get(0)[3] = (data1.size() - sc.size()) + ".0";
	}

	public static void fillPaperData(List<SheetData> sheets, TestPaper tp, List<String> header) {
		SheetData sheet = new SheetData();
		sheet.setHeader(header);
		sheet.setName("课程试卷信息");
		List<Object[]> data = new ArrayList<Object[]>();
		Object[] ob = new Object[header.size()];
		ob[0] = tp.getId();
		ob[1] = tp.getName();
		ob[2] = tp.getTestPaperContentId();
		ob[3] = tp.getScore();
		data.add(ob);
		sheet.setData(data);
		sheets.add(sheet);
	}

	private static void exportQuestion(Connection connect, String courseId, List<SheetData> sheets)
			throws SQLException, IOException {
		PreparedStatement preState = null;
		ResultSet resultSet = null;

		try {
			String sql = "SELECT t.question_type_enum qtype,q.ID id,t.question_type_enum dNumber,q.sequence number1,'1' score,q.PARENT_ID parent_id,q.QUESTION_TYPE_ENUM type,q.JSON"
					+ "	FROM QUESTION_CONTENT q,question_info t" + "	where t.id(+)=q.question_info_id"
					+ "   and  t.batch_type='1' and t.course_id='" + courseId
					+ "'	order by q.parent_id,t.question_type_enum,q.sequence";
			preState = connect.prepareStatement(sql);

			resultSet = preState.executeQuery();
			List<TestQuestion> tqs = new ArrayList<TestQuestion>();
			while (resultSet.next()) {
				TestQuestion tq = new TestQuestion();
				tq.setId(resultSet.getString("id"));
				tq.setdNumber(resultSet.getInt("dnumber"));
				tq.setdName(quesTypes.get(resultSet.getString("qtype")));
				tq.setNumber(resultSet.getInt("number1"));
				tq.setScore(resultSet.getDouble("score"));
				tq.setParentId(resultSet.getString("parent_id"));
				tq.setType(resultSet.getInt("type"));
				tq.setJson(resultSet.getString("json"));
				tqs.add(tq);
			}
			List<String> header = new ArrayList<String>();
			header.addAll(Arrays.asList(new String[] { "试题ID", "大题号", "大题名称", "小题号", "题分", "题父ID", "题类型" }));
			fillQuestionData(sheets, tqs, header);
		} finally {
			if (resultSet != null) {
				resultSet.close();
			}
			if (preState != null) {
				preState.close();
			}
		}
	}

	public static void fillQuestionData(List<SheetData> sheets, List<TestQuestion> list, List<String> header)
			throws IOException {
		SheetData sheet = new SheetData();
		sheet.setHeader(header);
		sheet.setName("试卷");
		List<Object[]> data = new ArrayList<Object[]>();
		for (TestQuestion b : list) {
			Object[] ob = new Object[header.size()];
			ob[0] = b.getId();
			ob[1] = b.getdNumber();
			ob[2] = b.getdName();
			ob[3] = b.getNumber();
			ob[4] = b.getScore();
			ob[5] = b.getParentId();
			ob[6] = b.getType();
			File txtFile = new File(quesDir + b.getId() + ".txt");
			if (!txtFile.exists()) {
				FileUtil.writeTxt(quesDir, b.getId(), b.getJson());
			}
			data.add(ob);
		}
		sheet.setData(data);
		sheets.add(sheet);
	}
}
