package cn.com.qmth.export;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFCellStyle;
import org.apache.poi.xssf.usermodel.XSSFRichTextString;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

/*
 * excel导出工具
 */
public class ExportUtils {
	private static Logger logger = LogManager.getLogger(ExportUtils.class);
	private static final String DEFALUT_EXT = ".xlsx";

	public static void exportExcel(String dir, String fileName, List<SheetData> data) {
		File file = new File(dir + fileName + DEFALUT_EXT);
		if (file.exists()) {
			file.delete();
		}
		OutputStream outputStream = null;
		try {
			outputStream = new FileOutputStream(file);
			write(data, outputStream);
			outputStream.flush();

		} catch (Exception e) {
			logger.error(e);
		} finally {
			if (outputStream != null) {
				try {
					outputStream.close();
				} catch (IOException e) {
					logger.error(e);
				}
			}
		}
	}

	private static void write(List<SheetData> data, OutputStream out) {
		// 声明一个工作薄
		XSSFWorkbook workbook = null;
		try {
			workbook = new XSSFWorkbook();
			for (SheetData sheetData : data) {
				// 生成一个表格
				XSSFSheet sheet = workbook.createSheet(sheetData.getName());
				// 生成一个样式
				XSSFCellStyle style = workbook.createCellStyle();

				// 产生表格标题行
				List<String> header = sheetData.getHeader();
				int[] colwidth = new int[header.size()];
				XSSFRow row = sheet.createRow(0);
				for (short i = 0; i < header.size(); i++) {
					colwidth[i] = 15;
					XSSFCell cell = row.createCell(i);
					cell.setCellStyle(style);
					XSSFRichTextString text = new XSSFRichTextString(header.get(i));
					cell.setCellValue(text);
					int textlen = length(header.get(i));
					if (textlen > colwidth[i]) {
						colwidth[i] = textlen;
					}
				}

				List<Object[]> rows = sheetData.getData();
				// 遍历集合数据，产生数据行
				for (short r = 0; r < rows.size(); r++) {
					row = sheet.createRow(r + 1);
					Object[] rowdata = rows.get(r);
					for (short i = 0; i < header.size(); i++) {
						XSSFCell cell = row.createCell(i);
						cell.setCellType(CellType.STRING);
						Object value = rowdata[i];
						// 判断值的类型后进行强制类型转换
						String textValue = null;
						if (value == null) {
							textValue = "";
						} else if (value instanceof Boolean) {
							boolean bValue = (Boolean) value;
							textValue = "是";
							if (!bValue) {
								textValue = "否";
							}
						} else if (value instanceof Date) {
							Date date = (Date) value;
							SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
							textValue = sdf.format(date);
						} else {
							// 其它数据类型都当作字符串简单处理
							textValue = String.valueOf(value);
						}
						cell.setCellValue(textValue);
						int textlen = length(textValue);
						if (textlen > colwidth[i]) {
							colwidth[i] = textlen;
						}
					}

				}
				for (int i = 0; i < colwidth.length; i++) {
					sheet.setColumnWidth(i, 286 * colwidth[i]);
				}
			}
			workbook.write(out);
		} catch (Exception e) {
			logger.error("导出出错", e);
		} finally {
			try {
				if (workbook != null) {
					workbook.close();
				}
			} catch (IOException e) {
				logger.error("导出出错", e);
			}
		}

	}

	private static int length(String value) {
		int valueLength = 0;
		String chinese = "[\u4e00-\u9fa5]";
		for (int i = 0; i < value.length(); i++) {
			String temp = value.substring(i, i + 1);
			if (temp.matches(chinese)) {
				valueLength += 2;
			} else {
				valueLength += 1;
			}
		}
		return valueLength;
	}
}
